/*
 * LynxMod.cpp
 *
 *  Created on: 22 f�vr. 2022
 *      Author: thomas.camguilhem
 */

#include "LynxMod.h"

#include <GCodes/GCodeBuffer/GCodeBuffer.h>

#if SUPPORT_SPI_SENSORS

#include <Platform/Tasks.h>
#include <Hardware/SharedSpi/SharedSpiDevice.h>


#if SUPPORT_LYNXMOD
// LYNXMOD:

#if SUPPORT_OBJECT_MODEL


// Object model table and functions
// Note: if using GCC version 7.3.1 20180622 and lambda functions are used in this table, you must compile this file with option -std=gnu++17.
// Otherwise the table will be allocated in RAM instead of flash, which wastes too much RAM.

// Macro to build a standard lambda function that includes the necessary type conversions
#define OBJECT_MODEL_FUNC(...) OBJECT_MODEL_FUNC_BODY(LynxMod, __VA_ARGS__)
#define OBJECT_MODEL_FUNC_IF(...) OBJECT_MODEL_FUNC_IF_BODY(LynxMod, __VA_ARGS__)

constexpr ObjectModelTableEntry LynxMod::objectModelTable[] =
{
};

constexpr uint8_t LynxMod::objectModelTableDescriptor[] = {
		0
};

DEFINE_GET_OBJECT_MODEL_TABLE(LynxMod)

#endif

LynxMod::LynxMod() noexcept :
device(SharedSpiDevice::GetMainSharedSpiDevice(), Nano_Frequency, Nano_SpiMode, NoPin, false)
{
}


void LynxMod::Init() noexcept
{
	listAmbiances[ 0] = (LightType){ "Ready\0", PRINT_READY};
	listAmbiances[ 1] = (LightType){ "Ready\0", PRINT_READY};
	listAmbiances[ 2] = (LightType){ "Idle\0", PRINT_IDLE_DC};
	listAmbiances[ 3] = (LightType){ "Idle\0", PRINT_IDLE_DO};
	listAmbiances[ 4] = (LightType){ "Print\0", PRINTING_DC};
	listAmbiances[ 5] = (LightType){ "Print\0", PRINTING_DO};
	listAmbiances[ 6] = (LightType){ "Paused\0", PRINT_PAUSED_DC};
	listAmbiances[ 7] = (LightType){ "Paused\0", PRINT_PAUSED_DO};
	listAmbiances[ 8] = (LightType){ "Done\0", PRINT_FINISHED}; // not implemented
	listAmbiances[ 9] = (LightType){ "Done\0", PRINT_FINISHED}; // not implemented

	listAmbiances[10] = (LightType){ "Error\0", ERREUR_MAJEURE_DC};
	listAmbiances[11] = (LightType){ "Error\0", ERREUR_MAJEURE_DO};
	listAmbiances[12] = (LightType){ "Warning\0", ERROR_NO_BIG_DEAL_DC};
	listAmbiances[13] = (LightType){ "Warning\0", ERROR_NO_BIG_DEAL_DO};
	listAmbiances[14] = (LightType){ "Update\0", UPDATE_FIRMWARE};
	listAmbiances[15] = (LightType){ "Update\0", UPDATE_FIRMWARE};
	listAmbiances[16] = (LightType){ "Flash\0", FLASH};
	listAmbiances[17] = (LightType){ "Flash\0", FLASH};

}


//GCodeResult RtdSensor31865::Configure(GCodeBuffer& gb, const StringRef& reply, bool& changed)
GCodeResult LynxMod::Configure(GCodeBuffer& gb, const StringRef& reply, bool& changed)
{
	if (!ConfigurePort(gb, reply, PinAccess::write1,changed))
	{
		return GCodeResult::error;
	}
	if (gb.Seen('F'))
	{
		changed = true;
	}

	if (gb.Seen('W'))
	{
		changed = true;
	}

	if (gb.Seen('R'))
	{
		changed = true;
	}

	if (gb.Seen('O'))
	{
		changed = true;
	}


	return FinishConfiguring(changed, reply);
}

//bool SpiTemperatureSensor::ConfigurePort(GCodeBuffer& gb, const StringRef& reply, bool& seen)
bool LynxMod::ConfigurePort(GCodeBuffer& gb, const StringRef& reply, PinAccess access, bool& seen)
{
	bool ret;
	if (gb.Seen('P')) {
		seen = true;
		ret = port.AssignPort(gb, reply, PinUsedBy::sensor, access);
	} else if (port.IsValid()) {
		ret = true;
	} else {
		reply.copy("{\"code\":3323,\"message\":\"Missing port name parameter\"}");
		ret = false;
	}
	device.SetCsPin(port.GetPin());
	return ret;
}

GCodeResult LynxMod::FinishConfiguring(bool changed, const StringRef& reply) noexcept
{
	if (changed)
	{
		// Initialise the sensor
		//InitSpi();
		lastReadingTime = millis();

		TemperatureError rslt;
		for (unsigned int i = 0; i < 3; ++i)		// try 3 times
		{
			rslt = TryInitLED();
			if (rslt == TemperatureError::success)
			{
				break;
			}
			delay(MinimumReadInterval);
		}

		lastReadingTime = millis();
		lastResult = rslt;

		if (rslt != TemperatureError::success)
		{
			reply.printf("{\"code\":3346,\"message\":\"Failed to initialise RTD: %s\",\"parameters\":{\"err\":%d}}", TemperatureErrorString(rslt), (int)rslt);
			return GCodeResult::error;
		}
	}
	return GCodeResult::ok;
}

TemperatureError LynxMod::TryInitLED() const noexcept
{
	const uint8_t modeData[2] = { 0x80, cr0 };			// write register 0
	uint32_t rawVal;
	TemperatureError sts = DoSpiTransaction(modeData, ARRAY_SIZE(modeData), rawVal);

	if (sts == TemperatureError::success)
	{
		delayMicroseconds(1);
		static const uint8_t readData[2] = { 0x00, 0x00 };	// read register 0
		sts = DoSpiTransaction(readData, ARRAY_SIZE(readData), rawVal);
	}

	//debugPrintf("Status %d data %04x\n", (int)sts, (uint16_t)rawVal);
	if (sts == TemperatureError::success)
	{
		sts = TemperatureError::badResponse;
	}

	return sts;
}

TemperatureError LynxMod::DoSpiTransaction(const uint8_t dataOut[], size_t nbytes, uint32_t& rslt) const noexcept
{
	if (!device.Select(10))
	{
		return TemperatureError::busBusy;
	}

	delayMicroseconds(1);
	uint8_t rawBytes[8];
	const bool ok = device.TransceivePacket(dataOut, rawBytes, nbytes);
	delayMicroseconds(1);

	device.Deselect();
	delayMicroseconds(1);

	if (!ok)
	{
		return TemperatureError::timeout;
	}

	rslt = rawBytes[0];
	for (size_t i = 1; i < nbytes; ++i)
	{
		rslt <<= 8;
		rslt |= rawBytes[i];
	}

	return TemperatureError::success;
}

GCodeResult LynxMod::LynxM969(GCodeBuffer& gb, const StringRef& reply) THROWS(GCodeException) {
	if (gb.Seen('S')) { // Parametre S
		unsigned int val = gb.GetIValue();
		if (val == 6) {
			reply.catf(" M969 S6 Détecté\n");
			//M969 S2 P"spi.cs4"
			bool seen = false;
			return Configure(gb, reply, seen);
		}
		/*int pval = -1;
		float cval[5] = {-1.0, -1.0, -1.0, -1.0, -1.0};
		if (gb.Seen('P')) { // Parametre P
			pval = gb.GetFValue();
		}
		if (gb.Seen('R')) { // Parametre R
			cval[0] = gb.GetFValue();
			pval = (pval >= 0 ? pval : -2);
		}
		if (gb.Seen('V')) { // Parametre V
			cval[1] = gb.GetFValue();
			pval = (pval >= 0 ? pval : -2);
		}
		if (gb.Seen('B')) { // Parametre B
			cval[2] = gb.GetFValue();
			pval = (pval >= 0 ? pval : -2);
		}
		if (gb.Seen('W')) { // Parametre W
			cval[3] = gb.GetFValue();
			pval = (pval >= 0 ? pval : -2);
		}
		if (gb.Seen('L')) { // Parametre L
			cval[4]  = gb.GetFValue();
			pval = (pval >= 0 ? pval : -2);
		}
		LynxM969(val, pval, cval, reply);*/
	}
	return GCodeResult::ok;
}

#endif
#endif //SUPPORT_LYNXMOD
